#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-08-19
#
# monitor check point remote access vpn connections
#
# 2021-08-31: rewritten for CMK 2.0
# 2023-05-29: moved gui files to ~/local/lib/check_mk/gui/plugins/...
#
# info with policy server down and no remote access connections
# [[], [['', '0', '0', '0', '0', '2', 'Down', 'Policy Server is down']]]
#
# policy server activated/up no RA users
# [[], [['Policy Server', '6', '0', '4294967294', '0', '0', 'OK', 'Policy Server is up']]]
#
#      raUserState OBJECT-TYPE
#            SYNTAX  INTEGER {
#                          active(3),
#                          destroy(4),
#                          idle(129),
#                          phase1(130),
#                          down(131),
#                          init(132)
#             		}
#
#      raTunnelEncAlgorithm OBJECT-TYPE
#            SYNTAX  INTEGER {
#                          espDES(1),
#                          esp3DES(2),
#                          espCAST(5),
#                          esp3IDEA(7),
#                          espNULL(9),
#                          espAES128(129),
#                          espAES256(130)
#             		}
#
#      raTunnelAuthMethod OBJECT-TYPE
#            SYNTAX  INTEGER {
#                          preshared-key(1),
#                          dss-signature(2),
#                          rsa-signature(3),
#                          rsa-encryption(4),
#                          rev-rsa-encryption(5),
#                          xauth(129),
#                          crack(130)
#             		}
#
# CHECKPOINT-MIB::raUsersEntry.2.172.16.1.1.0 = STRING: "user1"
# CHECKPOINT-MIB::raUsersEntry.2.172.16.1.2.0 = STRING: "user2"
# CHECKPOINT-MIB::raUsersEntry.2.172.16.1.5.0 = STRING: "user3"
# CHECKPOINT-MIB::raUsersEntry.4.172.16.1.1.0 = Gauge32: 3
# CHECKPOINT-MIB::raUsersEntry.4.172.16.1.2.0 = Gauge32: 3
# CHECKPOINT-MIB::raUsersEntry.4.172.16.1.5.0 = Gauge32: 3
# CHECKPOINT-MIB::raExternalIpAddr.172.16.1.1.0 = IpAddress: 21.11.219.184
# CHECKPOINT-MIB::raExternalIpAddr.172.16.1.2.0 = IpAddress: 17.19.215.31
# CHECKPOINT-MIB::raExternalIpAddr.172.16.1.5.0 = IpAddress: 27.19.15.12
# CHECKPOINT-MIB::raUserState.172.16.1.1.0 = Wrong Type (should be INTEGER): Gauge32: 3
# CHECKPOINT-MIB::raUserState.172.16.1.2.0 = Wrong Type (should be INTEGER): Gauge32: 3
# CHECKPOINT-MIB::raUserState.172.16.1.5.0 = Wrong Type (should be INTEGER): Gauge32: 3
# CHECKPOINT-MIB::raOfficeMode.172.16.1.1.0 = Wrong Type (should be INTEGER): Gauge32: 1
# CHECKPOINT-MIB::raOfficeMode.172.16.1.2.0 = Wrong Type (should be INTEGER): Gauge32: 1
# CHECKPOINT-MIB::raOfficeMode.172.16.1.5.0 = Wrong Type (should be INTEGER): Gauge32: 1
# CHECKPOINT-MIB::raUseUDPEncap.172.16.1.1.0 = Wrong Type (should be INTEGER): Gauge32: 2
# CHECKPOINT-MIB::raUseUDPEncap.172.16.1.2.0 = Wrong Type (should be INTEGER): Gauge32: 2
# CHECKPOINT-MIB::raUseUDPEncap.172.16.1.5.0 = Wrong Type (should be INTEGER): Gauge32: 2
# CHECKPOINT-MIB::raVisitorMode.172.16.1.1.0 = Wrong Type (should be INTEGER): Gauge32: 0
# CHECKPOINT-MIB::raVisitorMode.172.16.1.2.0 = Wrong Type (should be INTEGER): Gauge32: 0
# CHECKPOINT-MIB::raVisitorMode.172.16.1.5.0 = Wrong Type (should be INTEGER): Gauge32: 0
# CHECKPOINT-MIB::raRouteTraffic.172.16.1.1.0 = Wrong Type (should be INTEGER): Gauge32: 1
# CHECKPOINT-MIB::raRouteTraffic.172.16.1.2.0 = Wrong Type (should be INTEGER): Gauge32: 1
# CHECKPOINT-MIB::raRouteTraffic.172.16.1.5.0 = Wrong Type (should be INTEGER): Gauge32: 1
# CHECKPOINT-MIB::raCommunity.172.16.1.1.0 = STRING: RemoteAccess
# CHECKPOINT-MIB::raCommunity.172.16.1.2.0 = STRING: RemoteAccess
# CHECKPOINT-MIB::raCommunity.172.16.1.5.0 = STRING: RemoteAccess
# CHECKPOINT-MIB::raTunnelEncAlgorithm.172.16.1.1.0 = Wrong Type (should be INTEGER): Gauge32: 130
# CHECKPOINT-MIB::raTunnelEncAlgorithm.172.16.1.2.0 = Wrong Type (should be INTEGER): Gauge32: 130
# CHECKPOINT-MIB::raTunnelEncAlgorithm.172.16.1.5.0 = Wrong Type (should be INTEGER): Gauge32: 130
# CHECKPOINT-MIB::raTunnelAuthMethod.172.16.1.1.0 = Wrong Type (should be INTEGER): Gauge32: 129
# CHECKPOINT-MIB::raTunnelAuthMethod.172.16.1.2.0 = Wrong Type (should be INTEGER): Gauge32: 129
# CHECKPOINT-MIB::raTunnelAuthMethod.172.16.1.5.0 = Wrong Type (should be INTEGER): Gauge32: 129
# CHECKPOINT-MIB::raLogonTime.172.16.1.1.0 = Wrong Type (should be INTEGER): Gauge32: 1565942591
# CHECKPOINT-MIB::raLogonTime.172.16.1.2.0 = Wrong Type (should be INTEGER): Gauge32: 1565935409
# CHECKPOINT-MIB::raLogonTime.172.16.1.5.0 = Wrong Type (should be INTEGER): Gauge32: 1565947344
# CHECKPOINT-MIB::raUsersEntry.30.172.16.1.1.0 = Gauge32: 8
# CHECKPOINT-MIB::raUsersEntry.30.172.16.1.2.0 = Gauge32: 8
# CHECKPOINT-MIB::raUsersEntry.30.172.16.1.5.0 = Gauge32: 8
#
# .1.3.6.1.4.1.2620.500.9000.1.2.172.16.1.1.0 = STRING: "user1"
# .1.3.6.1.4.1.2620.500.9000.1.2.172.16.1.2.0 = STRING: "user2"
# .1.3.6.1.4.1.2620.500.9000.1.2.172.16.1.5.0 = STRING: "user3"
# .1.3.6.1.4.1.2620.500.9000.1.4.172.16.1.1.0 = Gauge32: 3
# .1.3.6.1.4.1.2620.500.9000.1.4.172.16.1.2.0 = Gauge32: 3
# .1.3.6.1.4.1.2620.500.9000.1.4.172.16.1.5.0 = Gauge32: 3
# .1.3.6.1.4.1.2620.500.9000.1.19.172.16.1.1.0 = IpAddress: 21.11.219.184
# .1.3.6.1.4.1.2620.500.9000.1.19.172.16.1.2.0 = IpAddress: 17.19.215.31
# .1.3.6.1.4.1.2620.500.9000.1.19.172.16.1.5.0 = IpAddress: 27.19.15.12
# .1.3.6.1.4.1.2620.500.9000.1.20.172.16.1.1.0 = Gauge32: 3
# .1.3.6.1.4.1.2620.500.9000.1.20.172.16.1.2.0 = Gauge32: 3
# .1.3.6.1.4.1.2620.500.9000.1.20.172.16.1.5.0 = Gauge32: 3
# .1.3.6.1.4.1.2620.500.9000.1.21.172.16.1.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.500.9000.1.21.172.16.1.2.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.500.9000.1.21.172.16.1.5.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.500.9000.1.23.172.16.1.1.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.500.9000.1.23.172.16.1.2.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.500.9000.1.23.172.16.1.5.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.500.9000.1.24.172.16.1.1.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.500.9000.1.24.172.16.1.2.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.500.9000.1.24.172.16.1.5.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.500.9000.1.25.172.16.1.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.500.9000.1.25.172.16.1.2.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.500.9000.1.25.172.16.1.5.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.500.9000.1.26.172.16.1.1.0 = STRING: "RemoteAccess"
# .1.3.6.1.4.1.2620.500.9000.1.26.172.16.1.2.0 = STRING: "RemoteAccess"
# .1.3.6.1.4.1.2620.500.9000.1.26.172.16.1.5.0 = STRING: "RemoteAccess"
# .1.3.6.1.4.1.2620.500.9000.1.27.172.16.1.1.0 = Gauge32: 130
# .1.3.6.1.4.1.2620.500.9000.1.27.172.16.1.2.0 = Gauge32: 130
# .1.3.6.1.4.1.2620.500.9000.1.27.172.16.1.5.0 = Gauge32: 130
# .1.3.6.1.4.1.2620.500.9000.1.28.172.16.1.1.0 = Gauge32: 129
# .1.3.6.1.4.1.2620.500.9000.1.28.172.16.1.2.0 = Gauge32: 129
# .1.3.6.1.4.1.2620.500.9000.1.28.172.16.1.5.0 = Gauge32: 129
# .1.3.6.1.4.1.2620.500.9000.1.29.172.16.1.1.0 = Gauge32: 1565942591
# .1.3.6.1.4.1.2620.500.9000.1.29.172.16.1.2.0 = Gauge32: 1565935409
# .1.3.6.1.4.1.2620.500.9000.1.29.172.16.1.5.0 = Gauge32: 1565947344
# .1.3.6.1.4.1.2620.500.9000.1.30.172.16.1.1.0 = Gauge32: 8
# .1.3.6.1.4.1.2620.500.9000.1.30.172.16.1.2.0 = Gauge32: 8
# .1.3.6.1.4.1.2620.500.9000.1.30.172.16.1.5.0 = Gauge32: 8
#
# string_table = [
#     ['user1', '3', '21.11.219.184', '3', '1', '', '2', '0', '1', 'RemoteAccess', '130', '129', '1565942591'],
#     ['user2', '3', '17.19.215.31', '3', '1', '', '2', '0', '1', 'RemoteAccess', '130', '129', '1565935409'],
#     ['user3', '3', '27.19.15.12', '3', '1', '', '2', '0', '1', 'RemoteAccess', '130', '129', '1565947344']
# ]
#

import time
from dataclasses import dataclass
from typing import List, Optional, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    check_levels,
    Metric,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


class BadUser(NamedTuple):
    user: str
    remote_ip: str
    enc_algorithm: str
    logon_timne: str


@dataclass
class CheckpointRemoteaccess:
    raUsers: int
    raUsersActive: int
    raUsersDestroy: int
    raUsersIdle: int
    raUsersPhase1: int
    raUsersDown: int
    raUsersInit: int
    raUsersOfficeMode: int
    raUsersIkeOverTcp: int
    raUsersUDPEncap: int
    raUsersVisitorMode: int
    raBadEncryption: int
    badUsers: Optional[List[BadUser]] = None


_raTunnelEncAlgorithm = {
    '1': 'DES',
    '2': '3DES',
    '5': 'CAST',
    '7': '3IDEA',
    '9': 'NULL',
    '129': 'AES128',
    '130': 'AES256',
}


def parse_checkpoint_remoteaccess(string_table: StringTable) -> Optional[CheckpointRemoteaccess]:
    remote_access = CheckpointRemoteaccess(
        raUsers=len(string_table),
        raUsersActive=0,
        raUsersDestroy=0,
        raUsersIdle=0,
        raUsersPhase1=0,
        raUsersDown=0,
        raUsersInit=0,
        raUsersOfficeMode=0,
        raUsersIkeOverTcp=0,
        raUsersUDPEncap=0,
        raUsersVisitorMode=0,
        raBadEncryption=0,
        badUsers=[],
    )

    for entry in string_table:
        try:
            raUserName, raExternalIpAddr, raUserState, raOfficeMode, raIkeOverTCP, raUseUDPEncap, \
            raVisitorMode, raTunnelEncAlgorithm, raTunnelAuthMethod, raLogonTime = entry
        except ValueError:
            return
        raUserState = int(raUserState)

        if raUserState == 3:
            remote_access.raUsersActive += 1
        elif raUserState == 4:
            remote_access.raUsersDestroy += 1
        elif raUserState == 129:
            remote_access.raUsersIdle += 1
        elif raUserState == 130:
            remote_access.raUsersPhase1 += 1
        elif raUserState == 131:
            remote_access.raUsersDown += 1
        elif raUserState == 132:
            remote_access.raUsersInit += 1

        if raOfficeMode == '1':
            remote_access.raUsersOfficeMode += 1

        if raIkeOverTCP == '1':
            remote_access.raUsersIkeOverTcp += 1

        if raUseUDPEncap == '2':
            remote_access.raUsersUDPEncap += 1

        if raVisitorMode == '1':
            remote_access.raUsersVisitorMode += 1

        if int(raTunnelEncAlgorithm) in [1, 2, 5, 7, 9]:
            remote_access.raBadEncryption += 1
            remote_access.badUsers.append(BadUser(
                user=raUserName,
                remote_ip=raExternalIpAddr,
                enc_algorithm=_raTunnelEncAlgorithm.get(raTunnelEncAlgorithm),
                logon_timne=time.ctime(int(raLogonTime)),
            ))

    return remote_access
    

def discovery_checkpoint_remoteaccess(section: CheckpointRemoteaccess) -> DiscoveryResult:
    yield Service()


def check_checkpoint_remoteaccess(params, section: CheckpointRemoteaccess) -> CheckResult:
    yield from check_levels(
        value=section.raUsers,
        label='Users',
        levels_upper=params.get('levels_upper_ra_users'),
        render_func=lambda v: f'{v:.0f}',
        boundaries=(0, None),
        metric_name='checkpoint_remoteaccess_raUsers',
    )

    if section.raBadEncryption > 0:
        yield Result(state=State.WARN, notice=f'{section.raBadEncryption} user(s) with bad encryption')
        for bad_user in section.badUsers:
            yield Result(
                state=State.OK,
                notice=f'User: {bad_user.user}, '
                       f'Remote IP: {bad_user.remote_ip}, '
                       f'encryption: {bad_user.enc_algorithm}, '
                       f'logon time: {bad_user.logon_timne}'
            )

    for key, value in [
                ('raUsersActive', section.raUsersActive),
                ('raUsersDestroy', section.raUsersDestroy),
                ('raUsersIdle', section.raUsersIdle),
                ('raUsersPhase1', section.raUsersPhase1),
                ('raUsersDown', section.raUsersDown),
                ('raUsersInit', section.raUsersInit),
                ('raBadEncryption', section.raBadEncryption),
                ('raUsersOfficeMode', section.raUsersOfficeMode),
                ('raUsersIkeOverTcp', section.raUsersIkeOverTcp),
                ('raUsersUDPEncap', section.raUsersUDPEncap),
                ('raUsersVisitorMode', section.raUsersVisitorMode),
               ]:
        yield Metric(
            value=value,
            name=f'checkpoint_remoteaccess_{key}',
            boundaries=(0, None),
        )


register.snmp_section(
    name='checkpoint_remoteaccess',
    parse_function=parse_checkpoint_remoteaccess,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.500.9000.1',  #
        oids=[
            '2',  # UserName
            '19',  # raExternalIpAddr
            '20',  # raUserState
            '21',  # raOfficeMode
            '22',  # raIkeOverTCP
            '23',  # raUseUDPEncap
            '24',  # raVisitorMode
            '27',  # raTunnelEncAlgorithm
            '28',  # raTunnelAuthMethod
            '29',  # raLogonTime
        ]
    ),
    detect=all_of(
        any_of(
            startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
            all_of(
                equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
                equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
            )
        ),
        equals('.1.3.6.1.4.1.2620.1.1.21.0', 'Firewall'),
    )
)

register.check_plugin(
    name='checkpoint_remoteaccess',
    service_name='Remote Access',
    discovery_function=discovery_checkpoint_remoteaccess,
    check_function=check_checkpoint_remoteaccess,
    check_ruleset_name='checkpoint_remoteaccess',
    check_default_parameters={},
)
