#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-08-19
#
# Check Point Policy Server connections metrics plugins
# checkpoint_remoteaccess
#
from cmk.gui.i18n import _

from cmk.gui.plugins.metrics.utils import (
    metric_info,
    graph_info,
    perfometer_info
)

metric_info['checkpoint_remoteaccess_raUsers'] = {
    'title': _('RA Connected users'),
    'unit': 'count',
    'color': '26/a',
}

metric_info['checkpoint_remoteaccess_raUsersActive'] = {
    'title': _('RA users active'),
    'unit': 'count',
    'color': '12/a',
}
metric_info['checkpoint_remoteaccess_raUsersDestroy'] = {
    'title': _('RA users destroy'),
    'unit': 'count',
    'color': '22/a',
}
metric_info['checkpoint_remoteaccess_raUsersIdle'] = {
    'title': _('RA users idle'),
    'unit': 'count',
    'color': '32/a',
}
metric_info['checkpoint_remoteaccess_raUsersPhase1'] = {
    'title': _('RA users phase1'),
    'unit': 'count',
    'color': '42/a',
}
metric_info['checkpoint_remoteaccess_raUsersDown'] = {
    'title': _('RA users down'),
    'unit': 'count',
    'color': '13/a',
}
metric_info['checkpoint_remoteaccess_raUsersInit'] = {
    'title': _('RA users init'),
    'unit': 'count',
    'color': '23/a',
}

metric_info['checkpoint_remoteaccess_raBadEncryption'] = {
    'title': _('RA users with bad encryption'),
    'unit': 'count',
    'color': '36/a',
}

metric_info['checkpoint_remoteaccess_raUsersOfficeMode'] = {
    'title': _('Office mode'),
    'unit': 'count',
    'color': '11/a',
}
metric_info['checkpoint_remoteaccess_raUsersIkeOverTcp'] = {
    'title': _('IKE over TCP'),
    'unit': 'count',
    'color': '21/a',
}
metric_info['checkpoint_remoteaccess_raUsersUDPEncap'] = {
    'title': _('UDP encapsulation'),
    'unit': 'count',
    'color': '31/a',
}

metric_info['checkpoint_remoteaccess_raUsersVisitorMode'] = {
    'title': _('Visitor mode'),
    'unit': 'count',
    'color': '41/a',
}

graph_info['checkpoint_remoteaccess_users'] = {
    'title': _('Check Point Remote Access Users'),
    'metrics': [
        ('checkpoint_remoteaccess_raUsers', 'line'),
        ('checkpoint_remoteaccess_raUsersActive', 'stack',),
        ('checkpoint_remoteaccess_raUsersDestroy', 'stack',),
        ('checkpoint_remoteaccess_raUsersIdle', 'stack',),
        ('checkpoint_remoteaccess_raUsersPhase1', 'stack',),
        ('checkpoint_remoteaccess_raUsersDown', 'stack',),
        ('checkpoint_remoteaccess_raUsersInit', 'stack',),
    ],
    'scalars': [
        ('checkpoint_remoteaccess_dtpsConnectedUsers:crit', _('crit')),
        ('checkpoint_remoteaccess_dtpsConnectedUsers:warn', _('warn')),
    ],
    'optional_metrics': [
        'checkpoint_remoteaccess_raUsers',
        'checkpoint_remoteaccess_raUsersActive',
        'checkpoint_remoteaccess_raUsersDestroy',
        'checkpoint_remoteaccess_raUsersIdle',
        'checkpoint_remoteaccess_raUsersPhase1',
        'checkpoint_remoteaccess_raUsersDown',
        'checkpoint_remoteaccess_raUsersInit',
    ],
    'range': (0, 'checkpoint_remoteaccess_raUsers:max'),
}

graph_info['checkpoint_remoteaccess_connection'] = {
    'title': _('Check Point Remote Access Users Connection'),
    'metrics': [
        ('checkpoint_remoteaccess_raUsersOfficeMode', 'line'),
        ('checkpoint_remoteaccess_raUsersIkeOverTcp', 'line',),
        ('checkpoint_remoteaccess_raUsersUDPEncap', 'line',),
        ('checkpoint_remoteaccess_raUsersVisitorMode', 'line',),
    ],

    'optional_metrics': [
        'checkpoint_remoteaccess_raUsersOfficeMode',
        'checkpoint_remoteaccess_raUsersIkeOverTcp',
        'checkpoint_remoteaccess_raUsersUDPEncap',
        'checkpoint_remoteaccess_raUsersVisitorMode',
    ],
}

graph_info['checkpoint_remoteaccess_raBadEncryption'] = {
    'title': _('Check Point Remote Access bad encryption'),
    'metrics': [
        ('checkpoint_remoteaccess_raBadEncryption', 'area'),
    ],
    'range': (0, 'checkpoint_remoteaccess_raBadEncryption:max'),
}

perfometer_info.append(
    {
        'type': 'logarithmic',
        'metric': 'checkpoint_remoteaccess_raUsers',
        'half_value': 500.0,
        'exponent': 2,
    },
)
